#!/bin/bash
#
#  generic routine for running blastall by splitting input and running
#  each on 1CPU, then reassembling at the end.  The input must be in 
#  a file.
#  parameters:
#  input file name
#  output file name
#  number of CPUs to use
#  all blast parameters including -d and -p but NOT -i or -o or -a
#
# There is NO error checking on the inputs!
# a file "input file name"_blast.log is also created.  
# If nothing goes wrong it will be empty.
#
# Dependencies:  Uses "extract" from drm_tools, available on sourceforge
#    https://sourceforge.net/projects/drmtools/
#
# David Mathog
# mathog@caltech.edu
# Version date: 2017/09/21
#
INFILE=$1
OUTFILE=$2
NCPUS=$3
BLASTPARAMS=$4
FPREFIX="${INFILE}_frag"
TEMPLATE="${FPREFIX}%3.3d"
OUTLOG="${INFILE}_blast.log"

NOW=`date`
echo "Infile $INFILE Outfile $OUTFILE NCPUS $NCPUS blast params $BLASTPARAMS"
echo "Splitting at $NOW"
ENTRIES=`fastaproperties $INFILE | extract -mt -fmt '[1]'`
if [ $ENTRIES -eq 0 ]
then
   echo "many_blast_plus_1cpu.sh: fatal error: exiting because empty input file: $INFILE"
   exit
fi

if [ $ENTRIES -lt $NCPUS ]
then
   echo "many_blast_plus_1cpu.sh: info: small input file, only using $ENTRIES threads"
   CHUNK=1
   NCPUS=$ENTRIES
else
   CHUNK=$(($ENTRIES / $NCPUS))
   TRIES=$(($CHUNK * $NCPUS))
   if [ $TRIES -ne $ENTRIES ]
   then
      CHUNK=$(( $CHUNK + 1 ))
   fi
fi
echo "Splitting $ENTRIES into $NCPUS chunks of size $CHUNK with template $TEMPLATE"
fastasplitn -in $INFILE -n $NCPUS -p 0 -c $CHUNK -t $TEMPLATE

#
#  If the number of entries is small compared to NCPUS some of the files created may be empty.
#  If this is true then there will be a frag000 file holding the number of frag files which
#  do have data.  Normally this script is called when this isn't the case, where there are a lot
#  of entries, where all output files are used.  Still, the case must be handled.
#
CFILE="${FPREFIX}000"
if [ -s $CFILE ]
then
   USED=`head -1 $CFILE`
   /bin/rm $CFILE
else
   USED=$NCPUS
fi

#
#  Start all the jobs, but only on input files which have data
#
STREAMS=$USED
I=1
while [ $I -le $STREAMS ]
do
   FRAG=`echo 000${I} | extract -fmt '[-3,-1]'`
   INFRAG="${FPREFIX}${FRAG}"
   OUTFRAG="${FPREFIX}${FRAG}.txt"
   LOGFRAG="${FPREFIX}${FRAG}.log"
   NOW=`date`
   echo "Starting blast for fragment $I $INFRAG at $NOW"
   nice blastall -i $INFRAG $BLASTPARAMS -a 1   >$OUTFRAG  2>$LOGFRAG  &
   I=$(($I + 1))
done

wait

# create empty output and log files, otherwise it would append
# to any existing file
cat >$OUTFILE <<EOD
EOD
cat >$OUTLOG <<EOD
EOD

STREAMS=$USED
I=1
while [ $I -le $STREAMS ]
do
   FRAG=`echo 000${I} | extract -fmt '[-3,-1]'`
   INLOG="${FPREFIX}${FRAG}.log"
   INFRAG="${FPREFIX}${FRAG}"
   INRESULT="${FPREFIX}${FRAG}.txt"
   NOW=`date`
   echo "merging blastall results $INRESULT at $NOW"
   cat $INLOG  >> $OUTLOG
   cat $INRESULT >> $OUTFILE
   rm -f $INLOG
   rm -f $INFRAG
   rm -f $INRESULT
   I=$(($I + 1))
done

# remove any remaining empty input files
while [ $I -le $NCPUS ]
do
   FRAG=`echo 000${I} | extract -fmt '[-3,-1]'`
   INFRAG="${FPREFIX}${FRAG}"
   rm -f $INFRAG
   I=$(($I + 1))
done

NOW=`date`
echo "many_blasts_1cpu.sh finished at $NOW"

